from pylama.lint import LinterV2 as Abstract
from pylama.context import RunContext

from pathlib import Path
from mypy import api
import re

regexes = {
    "dmypy_message": re.compile(
        r"(?P<filename>[^:]+):(?P<lnum>\d+):(?P<col>\d+): (?P<type>\w+): (?P<message>.+)"
    )
}


class Linter(Abstract):
    name = "dmypy"

    def run_check(self, ctx: RunContext):
        """Check code with dmypy."""
        args = ["run", "--", Path(ctx.temp_filename).parts[0], "--show-column-numbers"]
        stdout, a, b = api.run_dmypy(args)

        for line in stdout.splitlines():
            if not line:
                continue

            self.add_line(ctx, line, ctx.temp_filename)

    def add_line(self, ctx: RunContext, line: str, filename: str):
        m = regexes["dmypy_message"].match(line)
        if not m:
            return

        groups = m.groupdict()

        if groups["filename"] != filename:
            return

        lnum = int(groups["lnum"])
        col = int(groups["col"])
        mtype = groups["type"][0].upper()
        text = groups["message"].strip()

        ctx.push(source="dmypy", lnum=lnum, col=col, type=mtype, text=text)



from pylama_dmypy import VERSION

from setuptools import setup, find_packages

# fmt: off

setup(
      name = "pylama-dmypy"
    , version = VERSION
    , packages = find_packages(include="pylama_dmypy.*", exclude=["tests*"])

    , extras_require =
      { 'tests':
        [ "pylama==8.3.7"
        , "mypy==0.931"
        ]
      }

    , entry_points =
      { 'pylama.linter': ['dmypy = pylama_dmypy.linter:Linter']
      }

    # metadata
    , url = "http://github.com/delfick/pylama-dmypy"
    , author = "Stephen Moore"
    , author_email = "stephen@delfick.com"
    , description = "Linting plugin for pylama to see dmypy"
    , long_description = open("README.rst").read()
    , license = "MIT"
    )

# fmt: on
